@@ -4,8 +4,7 @@
 
 ;; Author: Milan Glacier <dev@milanglacier.com>
 ;; Maintainer: Milan Glacier <dev@milanglacier.com>
-;; Package-Version: 0.6.0
-;; Package-Revision: a6f4daf27e58
+;; Version: 0.6.0
 ;; URL: https://github.com/milanglacier/minuet-ai.el
 ;; Package-Requires: ((emacs "29") (plz "0.9") (dash "2.19.1"))
 
@@ -168,7 +167,7 @@ the cursor, the candidate will be truncated by those 20 characters
 before being delivered."
   :type 'integer)
 
-(defcustom minuet-before-cursor-filter-length 1
+(defcustom minuet-before-cursor-filter-length 2
   "Length of context before cursor used to filter completion text.
 
 This setting helps prevent the language model from generating
@@ -316,9 +315,9 @@ const processedData = transformData(rawData, {
     ,(cadr minuet-default-fewshots)))
 
 (defvar minuet-claude-options
-  `(:model "claude-3-5-haiku-20241022"
+  `(:model "claude-haiku-4-5"
     :end-point "https://api.anthropic.com/v1/messages"
-    :max_tokens 512
+    :max_tokens 256
     :api-key "ANTHROPIC_API_KEY"
     :system
     (:template minuet-default-system-template
@@ -337,6 +336,7 @@ const processedData = transformData(rawData, {
 (defvar minuet-openai-options
   `(:model "gpt-4.1-mini"
     :api-key "OPENAI_API_KEY"
+    :end-point "https://api.openai.com/v1/chat/completions"
     :system
     (:template minuet-default-system-template
      :prompt minuet-default-prompt
@@ -353,6 +353,7 @@ const processedData = transformData(rawData, {
 
 (defvar minuet-codestral-options
   '(:model "codestral-latest"
+    :name "Codestral"
     :end-point "https://codestral.mistral.ai/v1/fim/completions"
     :api-key "CODESTRAL_API_KEY"
     :template (:prompt minuet--default-fim-prompt-function
@@ -502,8 +503,8 @@ Also cancel any pending requests unless NO-CANCEL is t."
   (add-hook 'post-command-hook #'minuet--on-cursor-moved nil t)
   (when-let* ((suggestions suggestions)
               (cursor-not-moved (not (minuet--cursor-moved-p)))
-              (index (or index 0))
               (total (length suggestions))
+              (index (mod (or index 0) total))
               (suggestion (nth index suggestions))
               ;; 'Display' is used when not at the end-of-line to
               ;; ensure proper overlay positioning. Other methods,
@@ -571,7 +572,8 @@ Also cancel any pending requests unless NO-CANCEL is t."
   (let ((current-buffer (current-buffer))
         (available-p-fn (intern (format "minuet--%s-available-p" minuet-provider)))
         (complete-fn (intern (format "minuet--%s-complete" minuet-provider)))
-        (context (minuet--get-context)))
+        (context (minuet--get-context))
+        (is-first-completion t))
     (unless (funcall available-p-fn)
       (minuet--log (format "Minuet provider %s is not available" minuet-provider))
       (error "Minuet provider %s is not available" minuet-provider))
@@ -581,7 +583,9 @@ Also cancel any pending requests unless NO-CANCEL is t."
                (setq items (seq-uniq items))
                (with-current-buffer current-buffer
                  (when (and items (not (minuet--cursor-moved-p)))
-                   (minuet--display-suggestion items 0)))))))
+                   (minuet--display-suggestion
+                    items (if is-first-completion 0 minuet--current-suggestion-index))))
+               (setq is-first-completion nil)))))
 
 (defun minuet--log (message &optional message-p)
   "Log minuet messages into `minuet-buffer-name'.
@@ -712,7 +716,7 @@ conversation with alternating `user` and `assistant` roles by
     (nreverse results)))
 
 (cl-defun minuet--filter-text (item context)
-  "Filter ITEM based on CONTEXT using minuet-find-longest-match.
+  "Filter ITEM based on CONTEXT using `minuet-find-longest-match'.
 ITEM is a completion candidate string.
 CONTEXT is a plist with :before-cursor and :after-cursor fields.
 Returns the filtered item after trimming overlapping parts."
@@ -1046,8 +1050,7 @@ arrive."
            :then
            (lambda (json)
              (when-let* ((result (minuet--stream-decode json get-text-fn)))
-               ;; insert the current result into the completion items list
-               (push result completion-items))
+               (setq completion-items (append completion-items (list result))))
              (setq completion-items (minuet--filter-context-sequence-in-items
                                      completion-items
                                      context))
@@ -1059,7 +1062,7 @@ arrive."
                  (progn
                    (minuet--log (format "%s Request timeout" name))
                    (when-let* ((result (minuet--stream-decode-raw --response-- get-text-fn)))
-                     (push result completion-items)))
+                     (setq completion-items (append completion-items (list result)))))
                (minuet--log (format "An error occured when sending request to %s" name))
                (minuet--log err))
              (setq completion-items
@@ -1074,7 +1077,7 @@ arrive."
   "Complete code with codestral.
 CONTEXT and CALLBACK will be passed to the base function."
   (minuet--openai-fim-complete-base
-   (plist-put (copy-tree minuet-codestral-options) :name "Codestral")
+   minuet-codestral-options
    (plist-get minuet-codestral-options :get-text-fn)
    context
    callback))
@@ -1083,7 +1086,7 @@ CONTEXT and CALLBACK will be passed to the base function."
   "Complete code with openai fim API.
 CONTEXT and CALLBACK will be passed to the base function."
   (minuet--openai-fim-complete-base
-   (copy-tree minuet-openai-fim-compatible-options)
+   minuet-openai-fim-compatible-options
    (plist-get minuet-openai-fim-compatible-options :get-text-fn)
    context
    callback))
@@ -1161,16 +1164,12 @@ to be called when completion items arrive."
 (defun minuet--openai-complete (context callback)
   "Complete code with OpenAI.
 CONTEXT and CALLBACK will be passed to the base function."
-  (minuet--openai-complete-base
-   (--> (copy-tree minuet-openai-options)
-        (plist-put it :end-point "https://api.openai.com/v1/chat/completions"))
-   context callback))
+  (minuet--openai-complete-base minuet-openai-options context callback))
 
 (defun minuet--openai-compatible-complete (context callback)
   "Complete code with OpenAI compatible service.
 CONTEXT and CALLBACK will be passed to the base function."
-  (minuet--openai-complete-base
-   (copy-tree minuet-openai-compatible-options) context callback))
+  (minuet--openai-complete-base minuet-openai-compatible-options context callback))
 
 (defun minuet--claude-get-text-fn (json)
   "Function to get the completion from a JSON object for claude."
@@ -1192,7 +1191,7 @@ to be called when completion items arrive."
        :timeout minuet-request-timeout
        :body
        (json-serialize
-        (let ((options (copy-tree minuet-claude-options)))
+        (let ((options minuet-claude-options))
           `(,@(plist-get options :optional)
             :stream t
             :model ,(plist-get options :model)
@@ -1261,7 +1260,7 @@ to be called when completion items arrive."
        :timeout minuet-request-timeout
        :body
        (json-serialize
-        (let* ((options (copy-tree minuet-gemini-options))
+        (let* ((options minuet-gemini-options)
                (fewshots (minuet--eval-value (plist-get options :fewshots)))
                (fewshots (minuet--transform-openai-chat-to-gemini-chat fewshots)))
           `(,@(plist-get options :optional)
